@@ -1,11 +1,12 @@
 import time
+from concurrent.futures import ThreadPoolExecutor
 from datetime import date, datetime, timedelta
 from typing import Dict, List
 
 import alpaca_trade_api as tradeapi
+import pandas as pd
 import requests
 from alpaca_trade_api.common import get_polygon_credentials
-from alpaca_trade_api.polygon.entity import Ticker
 from asyncpg.pool import Pool
 from pandas import DataFrame as df
 from pandas import Timestamp
@@ -14,6 +15,7 @@ from pytz import timezone
 from liualgotrader.common import config, trading_data
 from liualgotrader.common.decorators import timeit
 from liualgotrader.common.tlog import tlog
+from liualgotrader.fincalcs.vwap import add_daily_vwap
 from liualgotrader.models.ticker_snapshot import TickerSnapshot
 
 volume_today: Dict[str, int] = {}
@@ -73,7 +75,7 @@ def get_historical_data_from_finnhub(symbols: List[str]) -> Dict[str, df]:
                         time.sleep(30)
                         retry = True
                     else:
-                        print(r.status_code, r.text)
+                        tlog(f"[ERROR] {r.status_code}, {r.text}")
 
         except KeyboardInterrupt:
             tlog("KeyboardInterrupt")
@@ -82,6 +84,118 @@ def get_historical_data_from_finnhub(symbols: List[str]) -> Dict[str, df]:
     return minute_history
 
 
+def get_historical_data_from_poylgon_for_symbols(
+    api: tradeapi, symbols: List[str], start_date: date, end_date: date
+) -> Dict[str, df]:
+    minute_history = {}
+    for symbol in symbols:
+        if symbol not in minute_history:
+            minute_history[symbol] = api.polygon.historic_agg_v2(
+                symbol,
+                1,
+                "minute",
+                _from=start_date,
+                to=end_date,
+            ).df.tz_convert("US/Eastern")
+            add_daily_vwap(minute_history[symbol])
+
+    return minute_history
+
+
+def get_historical_data_from_polygon_by_range(
+    api: tradeapi, symbols: List[str], start_date: date, timespan: str
+) -> Dict[str, df]:
+    """get ticker history"""
+
+    _minute_history: Dict[str, df] = {}
+    try:
+        for symbol in symbols:
+            from_date = start_date
+            while from_date < date.today():
+                retry = 5
+                _df = None
+                while retry > 0:
+                    try:
+                        _df = api.polygon.historic_agg_v2(
+                            symbol,
+                            1,
+                            timespan,
+                            _from=str(from_date),
+                            to=str(
+                                from_date
+                                + timedelta(
+                                    days=1 + config.polygon.MAX_DAYS_TO_LOAD
+                                )
+                            ),
+                        ).df
+                        break
+
+                    except Exception as e:
+
+                        retry -= 1
+
+                if _df is None or not len(_df):
+                    break
+
+                _df["vwap"] = 0.0
+                _df["average"] = 0.0
+
+                _minute_history[symbol] = (
+                    pd.concat([_minute_history[symbol], _df])
+                    if symbol in _minute_history
+                    else _df
+                )
+
+                from_date = _df.index[-1] + timedelta(days=1)
+
+                tlog(
+                    f"get_historical_data_from_polygon_by_range() - total loaded {len(_minute_history[symbol].index)} agg data points for {symbol}"
+                )
+    except KeyboardInterrupt:
+        tlog("KeyboardInterrupt")
+
+    return _minute_history
+
+
+def get_historical_daily_from_polygon_by_range(
+    api: tradeapi, symbols: List[str], start_date: date, end_date: date
+) -> Dict[str, df]:
+    """get ticker history"""
+
+    _minute_history: Dict[str, df] = {}
+    try:
+        for symbol in symbols:
+            retry = 5
+            _df = None
+            while retry > 0:
+                try:
+                    _df = api.polygon.historic_agg_v2(
+                        symbol,
+                        1,
+                        "day",
+                        _from=str(start_date),
+                        to=str(end_date),
+                    ).df
+
+                    _df["vwap"] = 0.0
+                    _df["average"] = 0.0
+
+                    _minute_history[symbol] = (
+                        pd.concat([_minute_history[symbol], _df])
+                        if symbol in _minute_history
+                        else _df
+                    )
+                    break
+                except Exception as e:
+                    retry -= 1
+                    continue
+
+    except KeyboardInterrupt:
+        tlog("KeyboardInterrupt")
+
+    return _minute_history
+
+
 def get_historical_data_from_polygon(
     api: tradeapi, symbols: List[str], max_tickers: int
 ) -> Dict[str, df]:
